{% load i18n l10n hijack %}

{% if request.user|can_hijack:another_user %}
  <button type="button" class="button" data-hijack-user="{{ another_user.pk|unlocalize }}"
          data-hijack-next="{{ next }}" data-hijack-url="{% url 'hijack:acquire' %}">
    {% if is_user_admin %}
      {% translate 'impersonate'|capfirst context 'verb' %}
    {% else %}
      {% blocktranslate trimmed %}
        impersonate {{ username }}
      {% endblocktranslate %}
    {% endif %}
  </button>
{% endif %}
